Attribute VB_Name = "Vitevitch"
'This module segregates the routines for running the Vitevitch phonotactic probability calculator utility.

        Option Explicit
    
    'Vitevitchian probabilities (not really probabilities, but proportions based on log of frequency).
        Public mUnigramPositionProbability() As Single   'Vitevitch probabilities - unigrams.  One entry per segment.
        Public mNumberOfBigrams As Long
        Public mBigramList() As String                   'A list of all bigrams
        Public mBigramPositionProbability() As Single    'Vitevitch probabilities - bigrams.

    'Bare segments are segments without designation of onset or coda position, as in the maxent grammar used here.
        Dim mNumberOfBareSegments As Long
        Dim mBareSegments() As String

Sub ReadVitevitchFiles()

    Dim MyLine As String, MySegment As String, MyBigram As String
    Dim BigramCounter As Long
    Dim SegmentIndex As Long, BigramIndex As Long, PositionIndex As Long
    
    'Compute mBareSegments()
        For SegmentIndex = 1 To BLICK.mNumberOfSegments
            If gSegments(SegmentIndex) <> "#" Then
                If Right(gSegments(SegmentIndex), 4) <> "Coda" Then
                    Let mNumberOfBareSegments = mNumberOfBareSegments + 1
                    ReDim Preserve mBareSegments(mNumberOfBareSegments)
                    Let mBareSegments(mNumberOfBareSegments) = gSegments(SegmentIndex)
                End If
            End If
        Next SegmentIndex
   
    'Redim the array with Vitevitch unigram frequencies.
        ReDim mUnigramPositionProbability(mNumberOfBareSegments, gLengthOfLongestWord) As Single
    
    'Digest the unigram file.
        Open App.Path + "/FilesNeededForOperation/VitevitchUnigramPercentages.txt" For Input As #1
            For SegmentIndex = 1 To mNumberOfBareSegments
                Line Input #1, MyLine
                Let MyLine = Trim(MyLine)
                Let MySegment = s.Chomp(MyLine)
                If MySegment <> mBareSegments(SegmentIndex) Then
                    MsgBox "Problem in the file VitevitchUnigramPercentages.txt.  The segment " + MySegment + " occurs where there ought to be " + gSegments(SegmentIndex) + "."
                End If
                'Read out the position files.
                    For PositionIndex = 1 To gLengthOfLongestWord
                       Let MyLine = s.Residue(MyLine)
                       Let mUnigramPositionProbability(SegmentIndex, PositionIndex) = Val(s.Chomp(MyLine))
                    Next PositionIndex
            Next SegmentIndex
        Close #1
        
    'Digest the bigram file.
        'Find out how many bigrams there are, so you can dimension the arrays.
            Open App.Path + "/FilesNeededForOperation/VitevitchBigramPercentages.txt" For Input As #1
                Do While Not EOF(1)
                    Line Input #1, MyLine
                    Let mNumberOfBigrams = mNumberOfBigrams + 1
                Loop
            Close #1
        'Redimension the bigram storage arrays.
            ReDim mBigramList(mNumberOfBigrams)
            ReDim mBigramPositionProbability(mNumberOfBigrams, gLengthOfLongestWord)
        'Reread the file to fill this array.
            Open App.Path + "/FilesNeededForOperation/VitevitchBigramPercentages.txt" For Input As #1
                Do While Not EOF(1)
                    Line Input #1, MyLine
                    Let BigramCounter = BigramCounter + 1
                    Let MyLine = Trim(MyLine)
                    Let mBigramList(BigramCounter) = s.Chomp(MyLine)
                    'Read out the position files.
                        For PositionIndex = 1 To gLengthOfLongestWord - 1
                           Let MyLine = s.Residue(MyLine)
                           Let mBigramPositionProbability(BigramCounter, PositionIndex) = Val(s.Chomp(MyLine))
                        Next PositionIndex
                Loop
            Close #1
        
        Exit Sub
        
    'Debug
        'Unigrams:
            Open App.Path + "/DebugVitevitchUnigrams.txt" For Output As #2
                For SegmentIndex = 1 To mNumberOfBareSegments
                    Print #2, mBareSegments(SegmentIndex);
                    For PositionIndex = 1 To gLengthOfLongestWord
                        Print #2, Chr(9); Str(mUnigramPositionProbability(SegmentIndex, PositionIndex));
                    Next PositionIndex
                    Print #2,
                Next SegmentIndex
            Close #2
        'Bigrams:
            Open App.Path + "/DebugVitevitchBigrams.txt" For Output As #2
                For BigramIndex = 1 To mNumberOfBigrams
                    Print #2, mBigramList(BigramIndex);
                    For PositionIndex = 1 To gLengthOfLongestWord - 1
                        Print #2, Chr(9); Str(mBigramPositionProbability(BigramIndex, PositionIndex));
                    Next PositionIndex
                    Print #2,
                Next BigramIndex
            Close #2
        End

End Sub

Function BareNumericalIndexOf(MyCharacter As String) As Long
    'Find the numerical index of a segment given as string; same as above but unsyllabified characters and no word boundary character.
        Dim SegmentIndex As Long
        For SegmentIndex = 1 To mNumberOfBareSegments
            If MyCharacter = mBareSegments(SegmentIndex) Then
                Let BareNumericalIndexOf = SegmentIndex
                Exit Function
            End If
        Next SegmentIndex
        'Failure:  report -1.
            Let BareNumericalIndexOf = -1
End Function

Public Function VitevitchSingleForm(MyWord As String)

    'Handle just one word at a time, input by the user on the interface.
        Dim MySavedWord As String, MyUnigramScore As Single, MyBigramScore As Single
        Dim UnigramReportResultsString As String, BigramReportResultsString As String
        
    'Clean up the word, remembering the original version.
        Let MyWord = BLICK.TrimCoda(MyWord, True)
        Let MySavedWord = MyWord
        
    'Run it past the error checker, syllabify it, announce the result, then compute the phonotactic score.
        If BLICK.ErrorCheck(MyWord, True) = "AllCharactersOK" Then
            Let MyUnigramScore = VitevitchUnigramScore(MyWord)
            Let UnigramReportResultsString = BLICK.mReportResultsString
            Let MyBigramScore = VitevitchBigramScore(MyWord)
            Let BigramReportResultsString = BLICK.mReportResultsString
            Let VitevitchSingleForm = "The unigram goodness score for this word is " + Trim(Str(MyUnigramScore)) + ", based on the positional values given below." + gLineBreak + Replace(UnigramReportResultsString, gSeparator, gLineBreak) + _
                gDoubleLineBreak + "The bigram goodness score for this word is " + Trim(Str(MyBigramScore)) + ", based on the positional values given below." + gLineBreak + Replace(BigramReportResultsString, gSeparator, gLineBreak)
        End If

End Function

Public Function VitevitchUnigramScore(CurrentWord As String)

    'Calculate the Vitevitch unigram score of a word.

    Dim MyWord As String, MySegment As String
    Dim ScoreBuffer As Single
    Dim PositionIndex As Long, MySegmentIndex As Long
    
    'Initialize the string that will report the violated constraints.
        Let BLICK.mReportResultsString = ""
    'Clean the word of extra spaces.
        Let MyWord = BLICK.CleanMe(CurrentWord)
    'Evaluate probabilities
        Do
            Let PositionIndex = PositionIndex + 1
            Let MySegment = s.CustomChomp(MyWord, " ")
            Let MySegmentIndex = BareNumericalIndexOf(MySegment)
            'Report failure for unrecognized segments.
                If MySegmentIndex = -1 Then
                    Let VitevitchUnigramScore = -1
                    Exit Function
                End If
            'Augment the score
                Let ScoreBuffer = ScoreBuffer + mUnigramPositionProbability(MySegmentIndex, PositionIndex)
            'Explain this
                Let BLICK.mReportResultsString = BLICK.mReportResultsString + gSeparator + MySegment
                Let BLICK.mReportResultsString = BLICK.mReportResultsString + " (" + Trim(Str(mUnigramPositionProbability(MySegmentIndex, PositionIndex))) + ") "
            Let MyWord = s.CustomResidue(MyWord, " ")
            If Trim(MyWord) = "" Then Exit Do
        Loop
    'Report outcome.
        Let VitevitchUnigramScore = ScoreBuffer

End Function

Public Function VitevitchBigramScore(CurrentWord As String)

    Dim MyWord As String, FirstSegment As String, SecondSegment As String, MyBigram As String
    Dim MyBigramProbability As Single, ScoreBuffer As Single
    Dim PositionIndex As Long, MyBigramIndex As Long
    
    'Initialize the string that will report the violated constraints.
        Let BLICK.mReportResultsString = ""
    'Clean the word of extra spaces and other crud.
        Let MyWord = BLICK.CleanMe(CurrentWord)
    'Grab the first segment, saving it for forming the first bigram.
        Let FirstSegment = s.CustomChomp(MyWord, " ")
        Let MyWord = s.CustomResidue(MyWord, " ")
    'Evaluate probabilities
        Do
            'Augment the location you are in.
                Let PositionIndex = PositionIndex + 1
            'Try to grab the next segment.
                Let SecondSegment = s.CustomChomp(MyWord, " ")
            'Quit if there actually are no more segments.
                If Trim(SecondSegment) = "" Then Exit Do
            'Form the current bigram and look up its index.
                Let MyBigram = FirstSegment + " " + SecondSegment
                Let MyBigramIndex = BigramIndexOf(MyBigram)
            'Proceed according to whether you found a real bigram or not.
                If MyBigramIndex = -1 Then
                    'Do nothing to the score -- in the Vitevitchian system, an unattested bigram simply fails to give the word credit.
                    'Explain this.
                        Let BLICK.mReportResultsString = BLICK.mReportResultsString + gSeparator + MyBigram + " (unattested)"
                Else
                    'You have an attested bigram. Look up its probability.
                        Let MyBigramProbability = mBigramPositionProbability(MyBigramIndex, PositionIndex)
                    'Augment the score
                        Let ScoreBuffer = ScoreBuffer + MyBigramProbability
                    'Explain the score.
                        Let BLICK.mReportResultsString = BLICK.mReportResultsString + gSeparator + MyBigram + " (" + Trim(Str(MyBigramProbability)) + ") "
                End If '
            'Move on to the rest of the word.
                Let MyWord = s.CustomResidue(MyWord, " ")
                Let FirstSegment = SecondSegment
        Loop
        
    'Report outcome
        Let VitevitchBigramScore = ScoreBuffer

End Function

Function BigramIndexOf(MyBigram As String) As Long
    'Look up the numerical index of a bigram, returning -1 if the bigram does not exist.
        Dim BigramIndex
        For BigramIndex = 1 To mNumberOfBigrams
            If mBigramList(BigramIndex) = MyBigram Then
                Let BigramIndexOf = BigramIndex
                Exit Function
            End If
        Next BigramIndex
        Let BigramIndexOf = -1
    
End Function

